use crate::entities::bpmn::YaSerialize;

use crate::entities::bpmn::datatypes::events::start_event::StartEvent;
use crate::entities::bpmn::datatypes::events::end_event::EndEvent;
use crate::entities::bpmn::datatypes::tasks::service_task::ServiceTask;
use crate::entities::bpmn::datatypes::flows::sequence_flow::SequenceFlow;

#[derive(Clone, Debug, YaSerialize, PartialEq)]
pub struct SubProcess {
    #[yaserde(attribute)]
    pub id: String,
    #[yaserde(attribute)]
    pub name: Option<String>,
    #[yaserde(child, rename = "bpmn2:startEvent")]
    pub start_event: Option<StartEvent>,
    #[yaserde(child, rename = "bpmn2:endEvent")]
    pub end_event: Option<EndEvent>,
    #[yaserde(child, rename = "bpmn2:serviceTask")]
    pub tasks: Vec<ServiceTask>,
    #[yaserde(child, rename = "bpmn2:sequenceFlow")]
    pub sequence_flows: Vec<SequenceFlow>
}

impl SubProcess {

    pub fn new_initial(id: String, name: String) -> SubProcess {
        SubProcess {
            id: id,
            name: Some(name.clone()),
            start_event:        None,
            end_event:          None,
            tasks:              Vec::new(),
            sequence_flows:     Vec::new()
        }
    }

    pub fn get_tasks(self) -> Vec<ServiceTask> {
        self.tasks
    }

    pub fn push_to_tasks(&mut self, service_task: ServiceTask) {
        self.tasks.push(service_task);
    }

    pub fn get_sequence_flows(self) -> Vec<SequenceFlow> {
        self.sequence_flows
    }

    pub fn push_to_flows(&mut self, sequence_flow: SequenceFlow) {
        self.sequence_flows.push(sequence_flow);
    }


}