use crate::entities::{bpmn, drawio_xml, messages};
use crate::domain::model::{UserJourney, Action};
use serde_json::error::Error as SerdeError;
use failure::Fail;

pub mod deserialize_input;
//pub mod generate_bpmn;
pub mod generate_json;
pub mod generate_user_journey;

#[derive(Fail, Debug)]
pub enum UseCaseExecError {
    #[fail(display = "The BPMN File could not be created. ({})", _0)]
    BPMNSerializationFailed(bpmn::BPMNError),
    #[fail(display = "The JSON File could not be created. ({})", _0)]
    JSONSerializationFailed(SerdeError),
    //#[fail(display = "The JSON File could not be deserialized. ({})", _0)]
    //WorkflowDeserializationFailed(workflow::WorkflowError),
    #[fail(display = "The Drawio File could not be deserialized. ({})", _0)]
    DrawioDeserializationFailed(drawio_xml::DrawioXMLError),
    #[fail(display = "Some Workflow Properties were not set.")]
    WorkflowPropertiesEmpty(),
    #[fail(display = "The MxGraphModel is invalid. ({})", _0)]
    MxGraphModelInvalid(String),
    #[fail(display = "The User Journey is abstract.")]
    UserJourneyAbstract()
}

// Workflow generation

/*pub fn generate_workflow_from_drawio_xml(input: String) -> Result<workflow::Workflow, UseCaseExecError> {
    let mx_graph_model = deserialize_input::to_mx_graph_model(input)?;
    generate_workflow::from_drawio_xml(&mx_graph_model)
}

pub fn generate_workflow_from_json(input: String) -> Result<workflow::Workflow, UseCaseExecError> {
    deserialize_input::to_workflow(input)
}*/

// UJSE Request generation

/*pub fn generate_ujse_request_from_workflow(workflow: workflow::Workflow) -> Result<messages::UJSERequest, UseCaseExecError> {
    let bpmn_string = generate_bpmn::from_workflow(&workflow)?;
    return Ok(messages::UJSERequest::new(workflow.get_origin_country(), workflow.get_destination_country(), workflow.get_workflow_type(), bpmn_string));
}*/

/*pub fn generate_ujse_request_from_user_journey(user_journey: UserJourney) -> Result<messages::UJSERequest, UseCaseExecError> {
    if user_journey.is_concrete() {
        let bpmn_string = generate_bpmn::from_user_journey(&user_journey)?;
        return Ok(messages::UJSERequest::new(user_journey.origin_country.unwrap(),
            user_journey.destination_country.unwrap(),
            user_journey.workflow_type.unwrap(),
            user_journey.description.unwrap_or("".to_string()),
            bpmn_string));
    } else {
        return Err(UseCaseExecError::UserJourneyAbstract());
    }
}*/

pub fn generate_mark_response_from_drawio(input: String) -> Result<messages::MarkResponse, UseCaseExecError>
{
    let mut res_touchpoints: Vec<messages::DrawioObject> = Vec::new();
    let mut res_actions: Vec<messages::DrawioObject> = Vec::new();

    let user_journey = generate_user_journey_from_drawio(input)?;

    let mut actions: Vec<&Action> = Vec::new();
    for phase in user_journey.phases.iter()
    {
        for action in phase.actions.iter()
        {
            actions.push(action);
        }
    }

    let mut a_index = 1; // for frontend display
    for action in actions.iter()
    {
        res_actions.push(messages::DrawioObject::new(action.get_drawio_id(), a_index));
        a_index += 1;

        let mut t_index = 0;
        for touchpoint in action.touchpoints.iter()
        {
            res_touchpoints.push(messages::DrawioObject::new(touchpoint.get_drawio_id(), t_index));
            t_index += 1;
        }
    }
    Ok(messages::MarkResponse::new(http::StatusCode::OK, user_journey.is_concrete(), res_touchpoints, res_actions))
}

// BPMN generation

/*pub fn generate_bpmn_from_drawio(input: String) -> Result<String, UseCaseExecError>
{
    let mx_graph_model = deserialize_input::to_mx_graph_model(input)?;
    let user_journey = generate_user_journey::from_drawio_xml(&mx_graph_model)?;
    if user_journey.is_concrete() {
        return generate_bpmn::from_user_journey(&user_journey);
    } else {
        return Err(UseCaseExecError::UserJourneyAbstract());
    }
}*/

// User Journey generation

pub fn generate_user_journey_from_drawio(input: String) -> Result<UserJourney, UseCaseExecError>
{
    let mx_graph_model = deserialize_input::to_mx_graph_model(input)?;
    generate_user_journey::from_drawio_xml(&mx_graph_model)
}

pub fn generate_user_journey_string_from_drawio(input: String) -> Result<String, UseCaseExecError>
{
    let mx_graph_model = deserialize_input::to_mx_graph_model(input)?;
    let user_journey = generate_user_journey::from_drawio_xml(&mx_graph_model)?;
    generate_json::from_user_journey(&user_journey)
        .map_err(|error| UseCaseExecError::JSONSerializationFailed(error))
}