use crate::domain::usecases;

use failure::Fail;
use std::io::Error;
use std::io::prelude::*;
use std::fs::File;
use std::path::PathBuf;

//mod to_bpmn;
mod to_json;

#[derive(Fail, Debug)]
pub enum PresenterError {
    #[fail(display = "The BPMN File could not be generated. ({})", _0)]
    GenerationFailed(usecases::UseCaseExecError),
    #[fail(display = "The Output File could not be created correctly. ({})", _0)]
    WriteFileFailed(Error)
}

/*pub fn to_bpmn_from_xml(input: String, out_dir: &str, out_file: &str) -> Result<(), PresenterError> {
    to_bpmn::from_drawio(input, out_dir, out_file)
}*/

pub fn to_json_from_xml(input: String, out_dir: &str, out_file: &str) -> Result<(), PresenterError> {
    to_json::from_drawio(input, out_dir, out_file)
}

fn write_file(data: &[u8], out_path: &PathBuf) -> Result<bool, Error> {
    let mut pos = 0;
    let mut buffer = File::create(out_path)?;
    while pos < data.len() {
        let bytes_written = buffer.write(&data[pos..])?;
        pos += bytes_written;
    }
    Ok(true)
}
